//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "sharedfilelist.h"
#include "knownfilelist.h"
#include "packets.h"
#include <time.h>


volatile BYTE		g_byHashThreadRunning;


CSharedFileList::CSharedFileList(CPreferences* pPrefs, CServerConnect* pServer, CKnownFileList* pFileList)
{
	g_byHashThreadRunning = 0;	
	::InitializeCriticalSection(&CFileHashThread::s_lockPartFileThread);

	m_pAppPrefs = pPrefs;
	m_pServer = pServer;
	m_pKnownFileList = pFileList;
	m_pOutput = 0;
	m_mapFilesKnown.InitHashTable(1229);// MSDN: best preformance if prime nr and 20% larger than the largest anticipated data set. 1229
	m_pThread = NULL;
	
	FindSharedFiles();
}

CSharedFileList::~CSharedFileList()
{
	if ((g_byHashThreadRunning&SHAREDFILESTHREAD) && m_pThread && m_pThread->m_hThreadHandle){
		TRACE("The App is shutting down, Entering wait for Thread: %d, @%d\n", m_pThread->m_hThreadHandle, ::GetTickCount());
		::WaitForSingleObject(m_pThread->m_hThreadHandle, INFINITE);
		TRACE("The App is shutting down, Wait successful for Thread: %d, @%d\n", m_pThread->m_hThreadHandle, ::GetTickCount());
		delete m_pThread;
	}

	::DeleteCriticalSection(&CFileHashThread::s_lockPartFileThread);
}

void CSharedFileList::FindSharedFiles()
{
	if (!m_mapFilesKnown.IsEmpty()){
		//CSingleLock sLock1(&list_mut,true); //use critical sections insted of mutexes...
		m_mapFilesKnown.RemoveAll();
		//sLock1.Unlock();
		theApp.downloadqueue->AddPartFilesToShare(); // readd partfiles
	}
	AddFilesFromDirectory(m_pAppPrefs->GetIncomingDir());
	
	for (POSITION pos = m_pAppPrefs->shareddir_list.GetHeadPosition(); pos != 0; m_pAppPrefs->shareddir_list.GetNext(pos))
		AddFilesFromDirectory(m_pAppPrefs->shareddir_list.GetAt(pos).GetBuffer());

	if (m_listWaitingForHash.IsEmpty())
		theApp.emuledlg->AddLogLine(false, GetResString(IDS_SHAREDFOUND), m_mapFilesKnown.GetCount());
	else if (!(g_byHashThreadRunning & SHAREDFILESTHREAD)){
		m_pThread = new CFileHashThread(&m_listWaitingForHash);
		m_pThread->BeginThread();
		g_byHashThreadRunning |= SHAREDFILESTHREAD;
		theApp.emuledlg->AddLogLine(false, GetResString(IDS_SHAREDFOUNDHASHING), m_mapFilesKnown.GetCount(), m_listWaitingForHash.GetCount());		
	}
}

void CSharedFileList::AddFilesFromDirectory(char* pszDirectory)
{
	//CSingleLock sLock(&list_mut);	//use critical sections insted of mutexes...
	bool bIdentical = false;

	CFileFind ff;
	char* pszSearchPath = new char[strlen(pszDirectory)+3];
	sprintf(pszSearchPath, "%s\\*", pszDirectory);
	bool bEnd = !ff.FindFile(pszSearchPath, 0);
	delete[] pszSearchPath;
	if (bEnd)
		return;
	//ff.FindNextFile();
	while (!bEnd){
		bEnd = !ff.FindNextFile();
		if (ff.IsDirectory() || ff.IsDots() || ff.IsSystem() || ff.IsTemporary())
			continue;
		CTime lwtime;
		ff.GetLastWriteTime(lwtime);
		uint32 nFileDate = mktime(lwtime.GetLocalTm());
		CKnownFile* pToAdd = m_pKnownFileList->FindKnownFile(ff.GetFileName().GetBuffer(), nFileDate, (uint32)ff.GetLength());
		if (pToAdd){
			pToAdd->SetPath(pszDirectory);
			//sLock.Lock();
			m_mapFilesKnown.SetAt(CCKey(pToAdd->GetFileHash()), pToAdd);
			//sLock.Unlock();
		}
		else{
			//not in knownfilelist
			UnknownFileStruct* pToHash = new UnknownFileStruct;
			pToHash->pszFileDirectory = nstrdup(pszDirectory);
			pToHash->pszFileName = nstrdup(ff.GetFileName().GetBuffer());

			if (!(g_byHashThreadRunning & SHAREDFILESTHREAD))
				m_listWaitingForHash.AddTail(pToHash);
			else{ // Probably is a better way of doing this...
				::EnterCriticalSection(&m_listWaitingForHash.m_lockFilePtrList);
				for (POSITION pos = m_listWaitingForHash.GetHeadPosition(); pos != 0; m_listWaitingForHash.GetNext(pos)){
					if ((_stricmp(m_listWaitingForHash.GetAt(pos)->pszFileName, pToHash->pszFileName) == 0) &&
						(_stricmp(m_listWaitingForHash.GetAt(pos)->pszFileDirectory, pToHash->pszFileDirectory) == 0)){
						bIdentical = true;
						delete[] pToHash->pszFileDirectory;
						delete[] pToHash->pszFileName;
						delete pToHash;
						break;
					}
				}
				::LeaveCriticalSection(&m_listWaitingForHash.m_lockFilePtrList);
				if (!bIdentical){
					m_listWaitingForHash.AddTail(pToHash);
					bIdentical = false;
				}
			}
		}
	}
	ff.Close();
}

void CSharedFileList::SafeAddKFile(CKnownFile* pToAdd, bool bOnlyAdd)
{
	// TODO: Check if the file is already known - only with another date
	//CSingleLock sLock(&list_mut,true);
	m_mapFilesKnown.SetAt(CCKey(pToAdd->GetFileHash()), pToAdd);
	//sLock.Unlock();
	if (m_listWaitingForHash.IsEmpty())
		m_pThread = NULL;
	if (bOnlyAdd)
		return;
	if (m_pOutput)
		m_pOutput->ShowFile(pToAdd);
	//HashNextFile();
	// offer new file to server
	if (!m_pServer->IsConnected())
		return;
	CMemFile* pFiles = new CMemFile(100);
	uint32 nFileCount = 1;
	pFiles->Write(&nFileCount, 4);
	CreateOfferedFilePacket(pToAdd, pFiles);
	Packet* pPacket = new Packet(pFiles);
	pPacket->opcode = OP_OFFERFILES;
	delete pFiles;
	m_pServer->SendPacket(pPacket, true);	
}

// removes first occurrence of 'toremove' in 'list'
void CSharedFileList::RemoveFile(CKnownFile* pToRemove)
{
	m_pOutput->RemoveFile(pToRemove);
	m_mapFilesKnown.RemoveKey(CCKey(pToRemove->GetFileHash()));
}

void CSharedFileList::Reload(bool bSendToServer)
{
	this->FindSharedFiles();
	if (m_pOutput)
		m_pOutput->ShowFileList(this);
	if (bSendToServer)
		SendListToServer();
}

void CSharedFileList::SetOutputCtrl(CSharedFilesCtrl* pCtrl)
{
	m_pOutput = pCtrl;
	m_pOutput->ShowFileList(this);
}

void CSharedFileList::SendListToServer(){
	if (m_mapFilesKnown.IsEmpty() || !m_pServer->IsConnected())
		return;
	CMemFile* pFiles = new CMemFile();
	uint32 nFileCount = m_mapFilesKnown.GetCount();
	pFiles->Write(&nFileCount, 4);
	
	CCKey bufKey;
	CKnownFile* pCurFile;
	for (POSITION pos = m_mapFilesKnown.GetStartPosition(); pos != 0; ){
		m_mapFilesKnown.GetNextAssoc(pos, bufKey, pCurFile);
		CreateOfferedFilePacket(pCurFile, pFiles);
	}
	Packet* pPacket = new Packet(pFiles);
	pPacket->opcode = OP_OFFERFILES;
	delete pFiles;
	m_pServer->SendPacket(pPacket, true);
}

void CSharedFileList::CreateOfferedFilePacket(CKnownFile* pCurFile, CMemFile* pFiles)
{
	pFiles->Write(pCurFile->GetFileHash(), 16);
	char* pszBuffer = new char[6];
	memset(pszBuffer, 0, 6);
	pFiles->Write(pszBuffer, 6);
	delete[] pszBuffer;
	pFiles->Write(pCurFile->GetFileTypePtr(), 4);
	CTag* pTagName = new CTag(FT_FILENAME, pCurFile->GetFileName());
	pTagName->WriteTagToFile(pFiles);
	delete pTagName;
	CTag* pTagSize = new CTag(FT_FILESIZE, pCurFile->GetFileSize());
	pTagSize->WriteTagToFile(pFiles);
	delete pTagSize;
	//TODO add tags for documents mp3 etc
}

uint64 CSharedFileList::GetDatasize()
{
	uint64 lSize;
	lSize=0;

	CCKey bufKey;
	CKnownFile* pCurFile;
	for (POSITION pos = m_mapFilesKnown.GetStartPosition(); pos != 0; ){
		m_mapFilesKnown.GetNextAssoc(pos, bufKey, pCurFile);
		lSize += pCurFile->GetFileSize();
	}
	return lSize;
}

CKnownFile*	CSharedFileList::GetFileByID(uchar* pszFileHash)
{
	CKnownFile* pResult;
	CCKey tkey(pszFileHash);
	if (m_mapFilesKnown.Lookup(tkey, pResult))
		return pResult;
	else
		return 0;
}

short CSharedFileList::GetFilePriorityByID(uchar* pszFileHash)
{
	CKnownFile* pToCheck = GetFileByID(pszFileHash);
	if (pToCheck)
		return pToCheck->GetPriority();
	else
		return -10;	// file doesn't exist
}

void CSharedFileList::UpdateItem(CKnownFile* pToUpdate)
{
	m_pOutput->UpdateItem(pToUpdate);
}

