//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#pragma once
#include "opcodes.h"
#include "emule.h"
#include "types.h"
#include "HashThread.h"
#include "preferences.h"
#include "KnownFile.h"
#include "knownfilelist.h"
#include "sharedfilesctrl.h"
#include "sockets.h"
#include "partfile.h"
#include "mapkey.h"


struct UnknownFileStruct
{
	char* pszFileName;
	char* pszFileDirectory;
};

class CKnownFileList;


// thread safe version of CTypedPtrList<CPtrList, UnknownFileStruct*>
class CFilePtrList : public CTypedPtrList<CPtrList, UnknownFileStruct*>
{
public:
	CFilePtrList() : CTypedPtrList<CPtrList, UnknownFileStruct*>()
						{ ::InitializeCriticalSection(&m_lockFilePtrList); };
	virtual ~CFilePtrList() 
						{ ::DeleteCriticalSection(&m_lockFilePtrList); };

	AFX_INLINE INT_PTR GetCount()	{ INT_PTR nResult;
						  ::EnterCriticalSection(&m_lockFilePtrList);
						  nResult = CTypedPtrList<CPtrList, UnknownFileStruct*>::GetCount();
						  ::LeaveCriticalSection(&m_lockFilePtrList);
						  return nResult; };
	
	AFX_INLINE bool IsEmpty()		{ bool bResult;
						  ::EnterCriticalSection(&m_lockFilePtrList);
						  bResult = CTypedPtrList<CPtrList, UnknownFileStruct*>::IsEmpty();
						  ::LeaveCriticalSection(&m_lockFilePtrList);
						  return bResult; };

	AFX_INLINE POSITION AddTail(UnknownFileStruct* pNewElement) 
						{ POSITION pos;
						  ::EnterCriticalSection(&m_lockFilePtrList);
						  pos = CTypedPtrList<CPtrList, UnknownFileStruct*>::AddTail(pNewElement);
						  ::LeaveCriticalSection(&m_lockFilePtrList);
						  return pos; };
	
	AFX_INLINE UnknownFileStruct* RemoveHead() 
						{ UnknownFileStruct* pFile;
						  ::EnterCriticalSection(&m_lockFilePtrList);
						  pFile = (UnknownFileStruct*)CTypedPtrList<CPtrList, UnknownFileStruct*>::RemoveHead();
						  ::LeaveCriticalSection(&m_lockFilePtrList);
						  return pFile; };

	CRITICAL_SECTION 	m_lockFilePtrList;
};


class CSharedFileList
{
	friend class CSharedFilesCtrl;
	friend class CClientReqSocket;
public:
	CSharedFileList(CPreferences* pPrefs, CServerConnect* pServer, CKnownFileList* pFileList);
	~CSharedFileList();
	
	void		SendListToServer();
	void		Reload(bool bSendToServer = true);
	void		SafeAddKFile(CKnownFile* pToAdd, bool bOnlyAdd = false);
	void		SetOutputCtrl(CSharedFilesCtrl* pCtrl);
	void		RemoveFile(CKnownFile* pToRemove);
	CKnownFile*	GetFileByID(uchar* pFileHash);
	short		GetFilePriorityByID(uchar* pFileHash);
	void		CreateOfferedFilePacket(CKnownFile* pCurFile, CMemFile* pFiles);
	uint64		GetDatasize();
	uint16		GetCount()	{ return m_mapFilesKnown.GetCount(); };
	void		UpdateItem(CKnownFile* pToUpdate);
	
	CKnownFileList*								m_pKnownFileList;

private:
	void		FindSharedFiles();
	void		AddFilesFromDirectory(char* pszDirectory);

	CMap<CCKey,CCKey&,CKnownFile*,CKnownFile*>	m_mapFilesKnown;
	CFilePtrList								m_listWaitingForHash;
	CPreferences*								m_pAppPrefs;
	CServerConnect*								m_pServer;
	CSharedFilesCtrl*							m_pOutput;
	CFileHashThread*							m_pThread;
};
